@@ -20,6 +20,8 @@ from .nonlinear import *
 from .operators_mpi import *
 from . import iterative
 from .iterative import pcg
+from .rules import rule_manager
+from .warnings import PyOperatorsWarning
 
 try:
     from .operators_pywt import *
@@ -27,10 +29,10 @@ except(ImportError):
     pass
 
 import types
-__all__ = [ f for f in dir() if f[0] != '_' and not isinstance(eval(f),
-            types.ModuleType)]
+__all__ = [f for f in dir() if f[0] != '_' and not isinstance(eval(f),
+           types.ModuleType)]
 
-del f #XXX not necessary with Python3
+del f  #XXX not necessary with Python3
 del types
 
-__version__ = '0.10.2'
+__version__ = '0.11'
