@@ -1,11 +1,15 @@
+#coding: utf-8
+from __future__ import division
+
 import numexpr
 if numexpr.__version__ < 2.0:
     raise ImportError('Please update numexpr to a newer version > 2.0.')
 
 import numpy as np
-from .decorators import idempotent, inplace, real, separable, square
-from .core import (Operator, BlockColumnOperator, CompositionOperator,
-                   IdentityOperator, ReductionOperator)
+from .core import (
+    Operator, BlockColumnOperator, CompositionOperator, ConstantOperator,
+    IdentityOperator, MultiplicationOperator, ReductionOperator)
+from .flags import idempotent, inplace, real, separable, square
 from .linear import DegreesOperator, RadiansOperator
 from .utils import (
     operation_assignment, operation_symbol, pi, strenum, tointtuple)
@@ -21,10 +25,14 @@ __all__ = ['Cartesian2SphericalOperator',
            'MinimumOperator',
            'NormalizeOperator',
            'NumexprOperator',
+           'PowerOperator',
            'ProductOperator',
+           'ReciprocalOperator',
            'RoundOperator',
            'SoftThresholdingOperator',
            'Spherical2CartesianOperator',
+           'SqrtOperator',
+           'SquareOperator',
            'To1dOperator',
            'ToNdOperator']
 
@@ -217,9 +225,9 @@ class ClipOperator(Operator):
 
     Arguments
     ---------
-    vmin: scalar or array_like
+    minvalue: scalar or array_like
         The minimum limit below which all input values are set to vmin.
-    vmax: scalar or array_like
+    maxvalue: scalar or array_like
         The maximum limit above which all input values are set to vmax.
 
     Exemples
@@ -236,9 +244,92 @@ class ClipOperator(Operator):
     MaximumOperator, MinimumOperator, np.clip
 
     """
-    def __init__(self, vmin, vmax, **keywords):
-        Operator.__init__(self, lambda i, o: np.clip(i, vmin, vmax, out=o),
-                          **keywords)
+    def __init__(self, minvalue, maxvalue, **keywords):
+        self.minvalue = np.asarray(minvalue)
+        self.maxvalue = np.asarray(maxvalue)
+        Operator.__init__(self, **keywords)
+
+    def direct(self, input, output):
+        np.clip(input, self.minvalue, self.maxvalue, out=output)
+
+    def __str__(self):
+        return 'clip(..., {0}, {1})'.format(self.minvalue, self.maxvalue)
+
+
+@square
+@inplace
+@separable
+class PowerOperator(Operator):
+    'X -> X**n'
+    def __init__(self, n, dtype=float, **keywords):
+        if np.allclose(n, -1) and not isinstance(self, ReciprocalOperator):
+            self.__class__ = ReciprocalOperator
+            self.__init__(dtype=dtype, **keywords)
+            return
+        if n == 0:
+            self.__class__ = ConstantOperator
+            self.__init__(1, dtype=dtype, **keywords)
+            return
+        if np.allclose(n, 0.5) and not isinstance(self, SqrtOperator):
+            self.__class__ = SqrtOperator
+            self.__init__(dtype=dtype, **keywords)
+            return
+        if np.allclose(n, 1):
+            self.__class__ = IdentityOperator
+            self.__init__(**keywords)
+            return
+        if np.allclose(n, 2) and not isinstance(self, SquareOperator):
+            self.__class__ = SquareOperator
+            self.__init__(dtype=dtype, **keywords)
+            return
+        self.n = n
+        Operator.__init__(self, dtype=dtype, **keywords)
+        self.set_rule('I', lambda s: PowerOperator(1/s.n))
+        self.set_rule(('.', PowerOperator),
+                      lambda s, o: PowerOperator(s.n * o.n),
+                      CompositionOperator)
+        self.set_rule(('.', PowerOperator),
+                      lambda s, o: PowerOperator(s.n + o.n),
+                      MultiplicationOperator)
+
+    def direct(self, input, output):
+        np.power(input, self.n, output)
+
+    def __str__(self):
+        return '...**{0}'.format(self.n)
+
+
+class ReciprocalOperator(PowerOperator):
+    'X -> 1 / X'
+    def __init__(self, **keywords):
+        PowerOperator.__init__(self, -1, **keywords)
+
+    def direct(self, input, output):
+        np.reciprocal(input, output)
+
+    def __str__(self):
+        return '1/...'
+
+
+class SqrtOperator(PowerOperator):
+    'X -> sqrt(X)'
+    def __init__(self, **keywords):
+        PowerOperator.__init__(self, 0.5, **keywords)
+
+    def direct(self, input, output):
+        np.sqrt(input, output)
+
+
+class SquareOperator(PowerOperator):
+    'X -> X**2'
+    def __init__(self, **keywords):
+        PowerOperator.__init__(self, 2, **keywords)
+
+    def direct(self, input, output):
+        np.square(input, output)
+
+    def __str__(self):
+        return u'...²'.encode('utf-8')
 
 
 class ProductOperator(ReductionOperator):
@@ -269,6 +360,10 @@ class ProductOperator(ReductionOperator):
         ReductionOperator.__init__(self, np.multiply, axis=axis, dtype=dtype,
                                    skipna=skipna, **keywords)
 
+    def __str__(self):
+        return 'product' if self.axis is None \
+                         else 'product(..., axis={0})'.format(self.axis)
+
 
 class MaxOperator(ReductionOperator):
     """
@@ -302,6 +397,10 @@ class MaxOperator(ReductionOperator):
         ReductionOperator.__init__(self, func, axis=axis, dtype=dtype,
                                    skipna=skipna, **keywords)
 
+    def __str__(self):
+        return 'max' if self.axis is None \
+                     else 'max(..., axis={0})'.format(self.axis)
+
 
 class MinOperator(ReductionOperator):
     """
@@ -335,6 +434,10 @@ class MinOperator(ReductionOperator):
         ReductionOperator.__init__(self, func, axis=axis, dtype=dtype,
                                    skipna=skipna, **keywords)
 
+    def __str__(self):
+        return 'min' if self.axis is None \
+                     else 'min(..., axis={0})'.format(self.axis)
+
 
 class MinMaxOperator(BlockColumnOperator):
     """
@@ -373,6 +476,10 @@ class MinMaxOperator(BlockColumnOperator):
         BlockColumnOperator.__init__(self, operands, new_axisout=new_axisout,
                                      **keywords)
 
+    def __str__(self):
+        return 'minmax' if self.axis is None \
+                        else 'minmax(..., axis={0})'.format(self.axis)
+
 
 @square
 @inplace
@@ -401,8 +508,14 @@ class MaximumOperator(Operator):
 
     """
     def __init__(self, value, **keywords):
-        Operator.__init__(self, lambda i, o: np.maximum(i, value, o),
-                          **keywords)
+        self.value = np.asarray(value)
+        Operator.__init__(self, **keywords)
+
+    def direct(self, input, output):
+        np.maximum(input, self.value, output)
+
+    def __str__(self):
+        return 'maximum(..., {0})'.format(self.value)
 
 
 @square
@@ -432,8 +545,14 @@ class MinimumOperator(Operator):
 
     """
     def __init__(self, value, **keywords):
-        Operator.__init__(self, lambda i, o: np.minimum(i, value, o),
-                          **keywords)
+        self.value = np.asarray(value)
+        Operator.__init__(self, **keywords)
+
+    def direct(self, input, output):
+        np.minimum(input, self.value, output)
+
+    def __str__(self):
+        return 'minimum(..., {0})'.format(self.value)
 
 
 @square
@@ -494,6 +613,9 @@ class NumexprOperator(Operator):
             expr = 'output' + op + '(' + self.expr + ')'
         numexpr.evaluate(expr, global_dict=self.global_dict, out=output)
 
+    def __str__(self):
+        return 'numexpr({0}, ...)'.format(self.expr)
+
 
 @square
 @idempotent
@@ -551,6 +673,16 @@ class RoundOperator(Operator):
         np.floor(output, output)
         np.add(output, 1, output)
 
+    def __str__(self):
+        method = self.method[1:]
+        if method == 'rmi':
+            method = 'floor'
+        elif method == 'tpi':
+            method = 'ceil'
+        elif method == 'tz':
+            method = 'trunc'
+        return 'round_{0}'.format(method)
+
 
 @square
 @idempotent
@@ -590,6 +722,9 @@ class HardThresholdingOperator(Operator):
     def direct(self, input, output):
         hard_thresholding(input, self.a, output)
 
+    def __str__(self):
+        return 'hardthreshold(..., {0})'.format(self.a)
+
 
 @square
 @inplace
@@ -624,6 +759,9 @@ class SoftThresholdingOperator(Operator):
     def direct(self, input, output):
         soft_thresholding(input, self.a, output)
 
+    def __str__(self):
+        return 'softthreshold(..., {0})'.format(self.a)
+
 
 @separable
 class _1dNdOperator(Operator):
@@ -741,3 +879,6 @@ class ToNdOperator(_1dNdOperator):
     def direct(self, input, output):
         np.floor_divide(input[..., None], self.coefs, out=output)
         np.mod(output, self.shape_, out=output)
+
+    def __str__(self):
+        return 'toNd'
