@@ -21,7 +21,7 @@ __all__ = ['assert_eq',
            'skiptest_unless_module']
 
 
-def assert_same(actual, desired, rtol=2, broadcasting=False):
+def assert_same(actual, desired, rtol=5, broadcasting=False):
     """
     Compare arrays of floats. The relative error depends on the data type.
 
@@ -40,15 +40,16 @@ def assert_same(actual, desired, rtol=2, broadcasting=False):
             "The actual array shape '{0}' is different from the desired one '{"
             "1}'.".format(actual.shape, desired.shape))
     dtype = sorted(arg.dtype for arg in [actual, desired])[0]
-    if dtype.kind in ('b', 'i'):
+    if dtype.kind in ('b', 'i', 'u'):
         if not broadcasting:
             assert_equal(actual, desired)
         else:
             assert np.all(actual == desired)
         return
     eps = np.finfo(dtype).eps * rtol
-    same = abs(actual - desired) <= eps * np.maximum(abs(actual), abs(desired))
+    same = abs(actual - desired) <= eps * np.minimum(abs(actual), abs(desired))
     same |= np.isnan(actual) & np.isnan(desired)
+    same |= actual == desired
     if not np.all(same):
         def trepr(x):
             r = repr(x).split('\n')
