@@ -1,29 +1,38 @@
 import itertools
 import numpy as np
 import operator
+import scipy
 import sys
 
+from nose import with_setup
 from nose.plugins.skip import SkipTest
-from pyoperators import memory, decorators
-from pyoperators.core import (
+from numpy.testing import assert_equal
+from pyoperators import memory, flags
+from pyoperators import (
     Operator, AdditionOperator, BlockColumnOperator, BlockDiagonalOperator,
-    BlockRowOperator, BlockSliceOperator, CompositionOperator, GroupOperator,
-    ConstantOperator, DenseOperator, DiagonalOperator, IdentityOperator,
-    MaskOperator, MultiplicationOperator, HomothetyOperator, ReductionOperator,
-    ZeroOperator, asoperator, _pool as pool, I, O)
+    BlockRowOperator, BlockSliceOperator, CompositionOperator,  GroupOperator,
+    ConstantOperator, DenseOperator, DiagonalOperator, HomothetyOperator,
+    IdentityOperator, MaskOperator, MultiplicationOperator, PowerOperator,
+    ReciprocalOperator, ReductionOperator, SparseOperator, SquareOperator,
+    ZeroOperator, asoperator, I, O, X)
+from pyoperators.core import CopyOperator, _pool as pool
 from pyoperators.memory import zeros
+from pyoperators.rules import rule_manager
 from pyoperators.utils import (
-    ndarraywrap, first_is_not, operation_assignment, product)
+    ndarraywrap, first_is_not, isalias, isscalarlike, operation_assignment,
+    product)
 from pyoperators.utils.mpi import MPI, distribute_slice
 from pyoperators.utils.testing import (
     assert_eq, assert_is, assert_is_not, assert_is_none, assert_not_in,
-    assert_is_instance, assert_raises, assert_same, skiptest)
+    assert_is_instance, assert_raises, assert_is_type, assert_same, skiptest)
+from scipy.sparse import csc_matrix
 from .common import OPS, ALL_OPS, DTYPES, HomothetyOutplaceOperator
 
 PYTHON_26 = sys.version_info < (2, 7)
 np.seterr(all='raise')
 
-memory.verbose = True
+old_memory_verbose = None
+old_memory_tolerance = None
 
 
 def assert_flags(operator, flags, msg=''):
@@ -71,14 +80,8 @@ class ndarray4(np.ndarray):
     pass
 
 
-@decorators.square
-class SquareOp(Operator):
-
-    def __init__(self, **keywords):
-        Operator.__init__(self, **keywords)
-
-
-@decorators.square
+@flags.linear
+@flags.square
 class Op2(Operator):
     attrout = {'newattr': True}
 
@@ -89,7 +92,8 @@ class Op2(Operator):
         pass
 
 
-@decorators.square
+@flags.linear
+@flags.square
 class Op3(Operator):
     classout = ndarray3
     classin = ndarray4
@@ -130,7 +134,7 @@ def test_flags():
             print 'Cannot test: ' + op.__name__
             return
         if o.flags.idempotent:
-            assert_is(o, o * o)
+            assert_is(o, o(o))
         if o.flags.real:
             assert_is(o, o.C)
         if o.flags.symmetric:
@@ -150,9 +154,8 @@ def test_flags():
 def test_symmetric():
     mat = np.matrix([[2, 1], [1, 2]])
 
-    @decorators.symmetric
+    @flags.symmetric
     class Op(Operator):
-
         def __init__(self):
             Operator.__init__(self, shapein=2, dtype=mat.dtype)
 
@@ -211,12 +214,30 @@ def test_autoflags():
         yield func, f
 
 
+#=============
+# Test direct
+#=============
+
+def test_ufuncs():
+    assert_raises(TypeError, Operator, np.maximum)
+
+    def func(ufunc, dtype):
+        o = Operator(np.cos)
+        assert_flags(o, 'real,inplace,outplace,square,separable')
+        assert o.dtype == dtype
+
+    ufuncs = np.cos, np.invert, np.negative
+    dtypes = np.float64, None, None
+    for ufunc, dtype in zip(ufuncs, dtypes):
+        yield func, ufunc, dtype
+
+
 #==================
 # Test conjugation
 #==================
 
 def test_conjugation():
-    @decorators.square
+    @flags.square
     class OpBase(Operator):
         def __init__(self, data_=None):
             Operator.__init__(self, shapein=2, dtype=complex)
@@ -289,6 +310,144 @@ def test_conjugation():
         yield func, opT, opH, opIT, opIH
 
 
+#==================
+# Test *, / and **
+#==================
+
+def test_times_mul_or_comp():
+    mat = [[1, 1, 1],
+           [0, 1, 1],
+           [0, 0, 1]]
+    ops = (2, [1, 2, 3], np.array(3), np.ones(3), np.negative, np.sqrt,
+           np.matrix(mat), csc_matrix(mat), DenseOperator(mat),
+           HomothetyOperator(3), SquareOperator(), X, X.T)
+
+    def islinear(_):
+        if isinstance(_, (np.matrix, csc_matrix)):
+            return True
+        if _ is np.sqrt:
+            return False
+        if _ is np.negative:
+            return True
+        if isscalarlike(_):
+            return True
+        return _.flags.linear
+
+    def func(x, y):
+        if np.__version__ < '1.9' and isinstance(x, np.ndarray):
+            if isinstance(x, np.matrix):
+                x = DenseOperator(x)
+            elif x.ndim > 0:
+                x = DiagonalOperator(x)
+        if scipy.__version__ < '0.14' and isinstance(x, csc_matrix):
+            x = SparseOperator(x)
+        if x is X.T and (y is np.sqrt or isinstance(y, SquareOperator)) or \
+           y is X.T and not isscalarlike(x) and \
+           not isinstance(x, HomothetyOperator):
+            assert_raises(TypeError, eval, 'x * y', {'x': x, 'y': y})
+            return
+
+        with rule_manager(none=True):
+            z = x * y
+
+        if x is X and y is X:
+            assert_is_type(z, MultiplicationOperator)
+        elif x is X.T and y is X or x is X and y is X.T:
+            assert_is_type(z, CompositionOperator)
+        elif x is X:
+            if np.isscalar(y) or \
+               isinstance(y, (list, np.ndarray, HomothetyOperator)) and \
+               not isinstance(y, np.matrix):
+                assert_is_type(z, CompositionOperator)
+            else:
+                assert_is_type(z, MultiplicationOperator)
+        elif type(x) is list or type(x) is np.ndarray and x.ndim > 0:
+            if y is X:
+                assert_is_type(z, CompositionOperator)
+            elif islinear(y):
+                assert_equal(z, asoperator(y).T(x))
+            else:
+                assert_is_type(z, MultiplicationOperator)
+        elif type(y) is list or type(y) is np.ndarray and y.ndim > 0:
+            if x is X.T:
+                assert_is_type(z, CompositionOperator)
+            elif islinear(x):
+                assert_equal(z, asoperator(x)(y))
+            else:
+                assert_is_type(z, MultiplicationOperator)
+        elif islinear(x) and islinear(y):
+            assert_is_type(z, CompositionOperator)
+        else:
+            assert_is_type(z, MultiplicationOperator)
+
+    for x in ops:
+        for y in ops:
+            if not isinstance(x, Operator) and not isinstance(y, Operator):
+                continue
+            yield func, x, y
+
+
+def test_div():
+    def func(flag):
+        op = 1 / Operator(flags={'linear': flag})
+        assert_is_type(op, CompositionOperator)
+        assert_is_type(op.operands[0], ReciprocalOperator)
+        assert_is_type(op.operands[1], Operator)
+    for flag in False, True:
+        yield func, flag
+
+
+def test_div_fail():
+    raise SkipTest
+    assert_is_type(1 / SquareOperator(), PowerOperator)
+
+
+def test_pow():
+    data = [[1, 1], [0, 1]]
+    op_lin = DenseOperator(data)
+    assert_equal((op_lin**3).data, np.dot(np.dot(data, data), data))
+    op_nl = ConstantOperator(data)
+    assert_equal((op_nl**3).data, data)
+
+
+def test_pow2():
+
+    @flags.linear
+    @flags.square
+    class SquareOp(Operator):
+        pass
+
+    def func(op, n):
+        p = op ** n
+        if n < -1:
+            assert_is_instance(p, CompositionOperator)
+            for o in p.operands:
+                assert_is(o, op.I)
+        elif n == -1:
+            assert_is(p, op.I)
+        elif n == 0:
+            assert_is_instance(p, IdentityOperator)
+        elif n == 1:
+            assert_is(p, op)
+        else:
+            assert_is_instance(p, CompositionOperator)
+            for o in p.operands:
+                assert_is(o, op)
+    for op in [SquareOp(), SquareOp(shapein=3)]:
+        for n in range(-3, 4):
+            yield func, op, n
+
+
+def test_pow3():
+    diag = np.array([1., 2, 3])
+    d = DiagonalOperator(diag)
+
+    def func(n):
+        assert_eq((d**n).todense(), DiagonalOperator(diag**n).todense())
+    for n in (-1.2, -1, -0.5, 0, 0.5, 1, 2.4):
+        yield func, n
+
+
 #========================
 # Test input/output shape
 #========================
@@ -307,9 +466,9 @@ def test_shape_is_inttuple():
 
 def test_shape_explicit():
     o1, o2, o3 = (
-        Operator(shapeout=(13, 2), shapein=(2, 2)),
-        Operator(shapeout=(2, 2), shapein=(1, 3)),
-        Operator(shapeout=(1, 3), shapein=4))
+        Operator(shapeout=(13, 2), shapein=(2, 2), flags='linear'),
+        Operator(shapeout=(2, 2), shapein=(1, 3), flags='linear'),
+        Operator(shapeout=(1, 3), shapein=4, flags='linear'))
 
     def func(o, eout, ein):
         assert_eq(o.shapeout, eout)
@@ -335,8 +494,8 @@ def test_shape_explicit():
 
 
 def test_shape_implicit():
+    @flags.linear
     class Op(Operator):
-
         def __init__(self, factor):
             self.factor = factor
             Operator.__init__(self)
@@ -370,8 +529,8 @@ def test_shapeout_unconstrained1():
 
 
 def test_shapeout_unconstrained2():
+    @flags.linear
     class Op(Operator):
-
         def direct(self, input, output):
             output[...] = 4
 
@@ -388,7 +547,6 @@ def test_shapeout_unconstrained2():
 
 def test_shapeout_implicit():
     class Op(Operator):
-
         def reshapein(self, shape):
             return shape + (2,)
 
@@ -416,7 +574,6 @@ def test_shapein_unconstrained1():
 
 def test_shapein_unconstrained2():
     class Op(Operator):
-
         def reshapeout(self, shape):
             return shape + (2,)
 
@@ -431,11 +588,11 @@ def test_shapein_unconstrained2():
 
 
 def test_shapein_unconstrained3():
-    @decorators.square
+    @flags.square
     class Op1(Operator):
         pass
 
-    @decorators.square
+    @flags.square
     class Op2(Operator):
         def reshapein(self, shape):
             return shape
@@ -443,7 +600,7 @@ def test_shapein_unconstrained3():
         def toshapein(self, v):
             return v
 
-    @decorators.square
+    @flags.square
     class Op3(Operator):
         def reshapeout(self, shape):
             return shape
@@ -451,7 +608,7 @@ def test_shapein_unconstrained3():
         def toshapeout(self, v):
             return v
 
-    @decorators.square
+    @flags.square
     class Op4(Operator):
         def reshapein(self, shape):
             return shape
@@ -521,9 +678,8 @@ def test_validation():
 def test_dtype1():
     value = 2.5
 
-    @decorators.square
+    @flags.square
     class Op(Operator):
-
         def __init__(self, dtype):
             Operator.__init__(self, dtype=dtype)
 
@@ -544,10 +700,11 @@ def test_dtype1():
         for di in DTYPES:
             yield func, dop, di
 
-def test_dtype2():
-    @decorators.square
-    class Op(Operator):
 
+def test_dtype2():
+    @flags.linear
+    @flags.square
+    class Op(Operator):
         def direct(self, input, output):
             np.multiply(input, input, output)
     op = Op()
@@ -586,7 +743,7 @@ def test_name():
 
 
 def test_merge_name():
-    @decorators.linear
+    @flags.linear
     class AbsorbOperator(Operator):
 
         def __init__(self, **keywords):
@@ -606,8 +763,8 @@ def test_merge_name():
         assert op.__name__ == name
     for (op, name), h in itertools.product(zip(ops, names),
                                            (I, HomothetyOperator(2))):
-        yield func, op * h, name
-        yield func, h * op, name
+        yield func, op(h), name
+        yield func, h(op), name
 
 
 #=========================
@@ -646,7 +803,8 @@ def test_iadd_imul():
 #===========================
 
 def test_propagation_attribute1():
-    @decorators.square
+    @flags.linear
+    @flags.square
     class AddAttribute(Operator):
         attrout = {'newattr_direct': True}
         attrin = {'newattr_transpose': True}
@@ -657,7 +815,8 @@ def test_propagation_attribute1():
         def transpose(self, input, output):
             pass
 
-    @decorators.square
+    @flags.linear
+    @flags.square
     class AddAttribute2(Operator):
         attrout = {'newattr_direct': False}
         attrin = {'newattr_transpose': False}
@@ -668,7 +827,8 @@ def test_propagation_attribute1():
         def transpose(self, input, output):
             pass
 
-    @decorators.square
+    @flags.linear
+    @flags.square
     class AddAttribute3(Operator):
         attrout = {'newattr3_direct': True}
         attrin = {'newattr3_transpose': True}
@@ -742,7 +902,7 @@ def test_propagation_attribute1():
 
 
 def test_propagation_attribute2():
-    @decorators.square
+    @flags.square
     class Op(Operator):
         attrin = {'attr_class': 1, 'attr_instance': 2, 'attr_other': 3}
         attrout = {'attr_class': 4, 'attr_instance': 5, 'attr_other': 6}
@@ -804,7 +964,8 @@ def test_propagation_attribute3():
             self.attr_class2 = 2
             self.attr_instance2 = 12
 
-    @decorators.square
+    @flags.linear
+    @flags.square
     class Op(Operator):
         classin = ndarray1
         classout = ndarray2
@@ -954,16 +1115,17 @@ def test_propagation_classT_inplace():
 
 
 def test_propagation_class_nested():
-    @decorators.square
+    @flags.linear
+    @flags.square
     class O1(Operator):
         classout = ndarray2
 
         def direct(self, input, output):
             output[...] = input
 
-    @decorators.square
+    @flags.linear
+    @flags.square
     class O2(Operator):
-
         def direct(self, input, output):
             output[...] = input
 
@@ -1111,8 +1273,8 @@ def test_comm_propagation():
         assert_is(opget.commin, commin)
         assert_is(opget.commout, commin)
     for cls in composite:
-        for i, ops in enumerate([(opgetcomm*Operator(), opsetcomm2),
-                                 (opsetcomm2, Operator()*opgetcomm)]):
+        for i, ops in enumerate([(opgetcomm(Operator()), opsetcomm2),
+                                 (opsetcomm2, Operator()(opgetcomm))]):
             keywords = {}
             if cls in (BlockDiagonalOperator, BlockRowOperator):
                 keywords = {'axisin': 0}
@@ -1138,7 +1300,7 @@ def test_comm_propagation():
             keywords = {'axisout': 0}
         for ops_in in [(opsetcomm2, Operator()), (Operator(), opsetcomm2)]:
             op_in = cls(ops_in, **keywords)
-            for i, op in enumerate([opgetcomm*op_in, op_in*opgetcomm]):
+            for i, op in enumerate([opgetcomm(op_in), op_in(opgetcomm)]):
                 yield func4, i, op
 
     # composite(get) + set in composition
@@ -1161,7 +1323,7 @@ def test_comm_propagation():
         for i, ops_in in enumerate([(opgetcomm, Operator()),
                                     (Operator(), opgetcomm)]):
             op_in = cls(ops_in, **keywords)
-            for j, op in enumerate([op_in*opsetcomm2, opsetcomm2*op_in]):
+            for j, op in enumerate([op_in(opsetcomm2), opsetcomm2(op_in)]):
                 yield func5, i, j, op
 
 
@@ -1170,9 +1332,8 @@ def test_comm_propagation():
 #===========================
 
 def test_inplace1():
-    @decorators.square
+    @flags.square
     class NotInplace(Operator):
-
         def direct(self, input, output):
             output[...] = 0
             output[0] = input[0]
@@ -1184,7 +1345,22 @@ def test_inplace1():
     assert_eq(len(pool), 1)
 
 
+def setup_memory():
+    global old_memory_tolerance, old_memory_verbose
+    old_memory_tolerance = memory.MEMORY_TOLERANCE
+    old_memory_verbose = memory.verbose
+    # ensure buffers in the pool are always used
+    memory.MEMORY_TOLERANCE = np.inf
+    memory.verbose = True
+
+
+def teardown_memory():
+    memory.MEMORY_TOLERANCE = old_memory_tolerance
+    memory.verbose = old_memory_verbose
+
+
 @skiptest
+@with_setup(setup_memory, teardown_memory)
 def test_inplace_can_use_output():
     A = zeros(10*8, dtype=np.int8).view(ndarraywrap)
     B = zeros(10*8, dtype=np.int8).view(ndarraywrap)
@@ -1202,18 +1378,20 @@ def test_inplace_can_use_output():
             self.log = log
 
         def direct(self, input, output):
+            if not self.inplace and isalias(input, output):
+                raise RuntimeError()
             if self.flags.inplace:
                 tmp = input[0]
                 output[1:] = 2 * input
                 output[0] = tmp
             else:
-                output[...] = 0
+                output[:] = 0
                 output[0] = input[0]
                 output[1:] = 2 * input
             try:
                 self.log.insert(0, ids[output.__array_interface__['data'][0]])
             except KeyError:
-                self.log.insert(0, 'unknown')
+                self.log.insert(0, '?')
 
         def reshapein(self, shape):
             return (shape[0]+1,)
@@ -1224,7 +1402,7 @@ def test_inplace_can_use_output():
             try:
                 result += ids[s.__array_interface__['data'][0]]
             except:
-                result += 'U'
+                result += '?'
         return result
 
     expecteds_outplace = {
@@ -1298,12 +1476,12 @@ def test_inplace_can_use_output():
         w = B[:(n+1)*8].view(float)
         op(v, w)
         log = ''.join(log) + 'A'
-        assert_eq(log, expected, strops)
-        assert_eq(show_pool(), 'CD', strops)
+        assert_equal(log, expected)
+        assert_equal(show_pool(), 'CD')
         w2 = v
         for op in reversed(ops):
             w2 = op(w2)
-        assert_eq(w, w2, strops)
+        assert_equal(w, w2)
 
     def func_inplace(n, i, expected, strops):
         pool._buffers = [B, C]
@@ -1315,15 +1493,13 @@ def test_inplace_can_use_output():
         w = A[:(n+1)*8].view(float)
         op(v, w)
         log = ''.join(log) + 'A'
-        assert_eq(log, expected, strops)
-        assert_eq(show_pool(), 'BC', strops)
+        assert_equal(log, expected)
+        assert_equal(show_pool(), 'BC')
         w2 = v
         for op in reversed(ops):
             w2 = op(w2)
-        assert_eq(w, w2, strops)
+        assert_equal(w, w2)
 
-    # prevent memory manager from allocating buffer: only use the provided pool
-    memory.MEMORY_TOLERANCE = np.inf
     for n in [2, 3, 4]:
         for i, expected in zip(reversed(range(2**n)), expecteds_outplace[n]):
             strops = bin(i)[2:]
@@ -1340,6 +1516,7 @@ def test_inplace_can_use_output():
 
 
 @skiptest
+@with_setup(setup_memory, teardown_memory)
 def test_inplace_cannot_use_output():
     A = np.zeros(10*8, dtype=np.int8).view(ndarraywrap)
     B = np.zeros(10*8, dtype=np.int8).view(ndarraywrap)
@@ -1353,16 +1530,19 @@ def test_inplace_cannot_use_output():
     class Op(Operator):
         def __init__(self, inplace, log):
             Operator.__init__(self, flags={'inplace': inplace})
+            self.inplace = inplace
             self.log = log
 
         def direct(self, input, output):
-            if not self.flags.inplace:
-                output[...] = 0
+            if not self.inplace and isalias(input, output):
+                raise RuntimeError()
+            if not self.inplace:
+                output[:] = 0
             output[:] = input[1:]
             try:
                 self.log.insert(0, ids[output.__array_interface__['data'][0]])
             except KeyError:
-                self.log.insert(0, 'unknown')
+                self.log.insert(0, '?')
 
         def reshapein(self, shape):
             return (shape[0]-1,)
@@ -1442,12 +1622,12 @@ def test_inplace_cannot_use_output():
         op(v, w)
         delattr(op, 'show_stack')
         log = ''.join(log) + 'A'
-        assert_eq(log, expected, strops)
-        assert_eq(show_stack(), 'CD', strops)
+        assert_equal(log, expected)
+        assert_equal(show_stack(), 'CD')
         w2 = v
         for op in reversed(ops):
             w2 = op(w2)
-        assert_eq(w, w2, strops)
+        assert_equal(w, w2)
 
     def func_inplace(n, i, expected, strops):
         pool._buffers = [B, C]
@@ -1461,12 +1641,12 @@ def test_inplace_cannot_use_output():
         op(v, w)
         delattr(op, 'show_stack')
         log = ''.join(log) + 'A'
-        assert_eq(log, expected, strops)
-        assert_eq(show_stack(), 'BC', strops)
+        assert_equal(log, expected)
+        assert_equal(show_stack(), 'BC')
         w2 = v
         for op in reversed(ops):
             w2 = op(w2)
-        assert_eq(w, w2, strops)
+        assert_equal(w, w2)
 
     for n in [2, 3, 4]:
         for i, expected in zip(reversed(range(2**n)), expecteds_outplace[n]):
@@ -1529,7 +1709,7 @@ def test_associativity():
     b = GroupOperator([Op3(), Op4()])
 
     def func3(o1, o2):
-        op = o1 * o2
+        op = o1(o2)
         assert_is_instance(op, CompositionOperator)
         assert_eq(len(op.operands), 2)
         assert_is(op.operands[0], o1)
@@ -1543,9 +1723,8 @@ def test_associativity():
 #==================
 
 def test_addition():
-    @decorators.square
+    @flags.square
     class Op(Operator):
-
         def __init__(self, v, **keywords):
             self.v = v
             Operator.__init__(self, **keywords)
@@ -1591,9 +1770,8 @@ def test_addition_flags():
 
 
 def test_multiplication():
-    @decorators.square
+    @flags.square
     class Op(Operator):
-
         def __init__(self, v, **keywords):
             self.v = v
             Operator.__init__(self, **keywords)
@@ -1731,7 +1909,7 @@ def test_composition1():
                 if shapeout is None and shapemid is not None:
                     continue
                 op2 = Operator(shapein=shapemid, shapeout=shapeout)
-                op = op2 * op1
+                op = op2(op1)
                 yield func, op, shapein, shapeout
 
 
@@ -1745,20 +1923,20 @@ def test_composition2():
         assert op.shapeout == (2*shape if shape is not None else None)
         assert_flags_false(op, 'square')
     for shape in SHAPES:
-        op = Op() * Operator(shapeout=shape)
+        op = Op()(Operator(shapeout=shape))
         yield func, op, shape
 
-    op = Op() * Op()
+    op = Op()(Op())
     assert op.shapein is None
     assert op.shapeout is None
     assert_flags_false(op, 'square')
 
 
 def test_composition3():
-    @decorators.square
-    @decorators.inplace
+    @flags.linear
+    @flags.square
+    @flags.inplace
     class Op(Operator):
-
         def __init__(self, v, **keywords):
             self.v = v
             Operator.__init__(self, **keywords)
@@ -1880,30 +2058,30 @@ def test_composition_shapes():
 
 
 def test_composition_get_requirements():
-    @decorators.inplace
+    @flags.inplace
     class I__(Operator):
         pass
 
-    @decorators.aligned
-    @decorators.contiguous
+    @flags.aligned
+    @flags.contiguous
     class IAC(I__):
         pass
 
     class O____(Operator):
         pass
 
-    @decorators.aligned_input
-    @decorators.contiguous_input
+    @flags.aligned_input
+    @flags.contiguous_input
     class O__AC(O____):
         pass
 
-    @decorators.aligned_output
-    @decorators.contiguous_output
+    @flags.aligned_output
+    @flags.contiguous_output
     class OAC__(O____):
         pass
 
-    @decorators.aligned
-    @decorators.contiguous
+    @flags.aligned
+    @flags.contiguous
     class OACAC(O____):
         pass
 
@@ -1956,41 +2134,17 @@ def test_composition_get_requirements():
             yield func, t, rn1, rn2, ra1, ra2, rc1, rc2
 
 
-#=============
-# Test powers
-#=============
+#====================
+# Test copy operator
+#====================
 
-def test_powers():
-
-    def func(op, n):
-        p = op ** n
-        if n < -1:
-            assert_is_instance(p, CompositionOperator)
-            for o in p.operands:
-                assert_is(o, op.I)
-        elif n == -1:
-            assert_is(p, op.I)
-        elif n == 0:
-            assert_is_instance(p, IdentityOperator)
-        elif n == 1:
-            assert_is(p, op)
-        else:
-            assert_is_instance(p, CompositionOperator)
-            for o in p.operands:
-                assert_is(o, op)
-    for op in [SquareOp(), SquareOp(shapein=3)]:
-        for n in range(-3, 4):
-            yield func, op, n
-
-
-def test_powers_diagonal():
-    diag = np.array([1., 2, 3])
-    d = DiagonalOperator(diag)
-
-    def func(n):
-        assert_eq((d**n).todense(), DiagonalOperator(diag**n).todense())
-    for n in (-1.2, -1, -0.5, 0, 0.5, 1, 2.4):
-        yield func, n
+def test_copy():
+    C = CopyOperator()
+    x = np.array([10, 20])
+    assert_equal(x, C(x))
+    x_ = x.copy()
+    C(x, x)
+    assert_equal(x, x_)
 
 
 #=============================
@@ -2158,7 +2312,7 @@ def test_homothety_reduction2():
 
 
 def test_homothety_reduction3():
-    @decorators.linear
+    @flags.linear
     class Op(Operator):
         pass
 
@@ -2214,7 +2368,7 @@ def test_constant_reduction2():
           ConstantOperator([1, 2, 3], broadcast='leftward'),
           ConstantOperator(np.ones((2, 3))))
     os = (I, H(2, shapein=(2, 3)) * Operator(direct=np.square, shapein=(2, 3),
-                                             flags='square'), H(5))
+                                             flags='linear,square'), H(5))
     results = (((H, 3), (C, (H, 6)), (H, 15)),
                ((D, [1, 2, 3]), (C, (D, [2, 4, 6])), (D, [5, 10, 15])),
                ((IdentityOperator, 1), (C, (H, 2)), (H, 5)))
@@ -2235,11 +2389,11 @@ def test_constant_reduction2():
 
 
 def _test_constant_reduction3():
-    @decorators.square
+    @flags.square
     class Op(Operator):
-
         def direct(self, input, output):
             output[...] = input + np.arange(input.size).reshape(input.shape)
+
     os = (Op(shapein=()), Op(shapein=4), Op(shapein=(2, 3, 4)))
     cs = (ConstantOperator(2), ConstantOperator([2], broadcast='leftward'),
           ConstantOperator(2*np.arange(8).reshape((2, 1, 4)),
@@ -2255,185 +2409,6 @@ def _test_constant_reduction3():
         yield func, o, c
 
 
-def test_zero1():
-    z = ZeroOperator()
-    o = Operator(shapein=3, shapeout=6)
-    zo = z*o
-    assert_is_instance(zo, ZeroOperator)
-    assert_eq(zo.shapein, o.shapein)
-    assert_is_none(zo.shapeout)
-
-
-def test_zero2():
-    z = ZeroOperator(shapein=3, shapeout=6)
-    o = Operator()
-    zo = z*o
-    assert_is_instance(zo, ZeroOperator)
-    assert_is_none(zo.shapein, 'in')
-    assert_eq(zo.shapeout, z.shapeout, 'out')
-
-
-def test_zero3():
-    z = ZeroOperator(shapein=3, shapeout=6)
-    o = Operator(flags='square')
-    zo = z*o
-    assert_is_instance(zo, ZeroOperator)
-    assert_eq(zo.shapein, z.shapein, 'in')
-    assert_eq(zo.shapeout, z.shapeout, 'out')
-
-
-def test_zero4():
-    z = ZeroOperator()
-    o = Operator(flags='linear')
-    assert_is_instance(z*o, ZeroOperator)
-    assert_is_instance(o*z, ZeroOperator)
-
-
-def test_zero5():
-    z = ZeroOperator()
-    o = Operator(shapein=3, shapeout=6, flags='linear')
-    zo = z*o
-    oz = o*z
-    assert_is_instance(zo, ZeroOperator, 'zo')
-    assert_eq(zo.shapein, o.shapein, 'zo in')
-    assert_is_none(zo.shapeout, 'zo out')
-    assert_is_instance(oz, ZeroOperator, 'oz')
-    assert_is_none(oz.shapein, 'oz, in')
-    assert_eq(oz.shapeout, o.shapeout, 'oz, out')
-
-
-def test_zero6():
-    z = ZeroOperator(flags='square')
-
-    @decorators.linear
-    class Op(Operator):
-        def direct(self, input, output):
-            output[:] = np.concatenate([input, 2*input])
-
-        def transpose(self, input, output):
-            output[:] = input[0:output.size]
-
-        def reshapein(self, shapein):
-            return (2 * shapein[0],)
-
-        def reshapeout(self, shapeout):
-            return (shapeout[0] // 2,)
-    o = Op()
-    zo = z*o
-    oz = o*z
-    v = np.ones(4)
-    assert_eq(zo.T(v), o.T(z.T(v)))
-    assert_eq(oz.T(v), z.T(o.T(v)))
-
-
-def test_zero7():
-    z = ZeroOperator()
-    assert_eq(z*z, z)
-
-
-def test_zero8():
-    class Op(Operator):
-        pass
-    o = Op()
-    assert type(o + O) is Op
-
-
-def test_dense():
-    def func(m, d, v):
-        expected = np.dot(m, v)
-        assert_eq(d(v), expected)
-        if d.flags.square:
-            w = v.copy()
-            d(w, w)
-            assert_eq(w, expected)
-
-    m = np.array([[1, 1j], [2, 2]])
-    d = DenseOperator(m)
-    for v in np.array([1+0j, 0]), np.array([0+0j, 1]):
-        yield func, m, d, v
-        yield func, m.T, d.T, v
-        yield func, m.T.conj(), d.H, v
-
-    m = np.array([[1, 2], [3, 4j], [5, 6]])
-    d = DenseOperator(m)
-    for v in np.array([1+0j, 0]), np.array([0+0j, 1]):
-        yield func, m, d, v
-    for v in (np.array([1+0j, 0, 0]), np.array([0j, 1, 0]),
-              np.array([0j, 0, 1])):
-        yield func, m.T, d.T, v
-        yield func, m.T.conj(), d.H, v
-
-
-def test_dense_rule_homothety():
-    m = np.array([[1, 2], [3, 4], [5, 6]])
-    d = HomothetyOperator(2) * DenseOperator(m)
-    assert_is_instance(d, DenseOperator)
-    assert_eq(d.data, m * 2)
-    d = HomothetyOperator(2j) * DenseOperator(m)
-    assert_is_instance(d, DenseOperator)
-    assert_eq(d.data, m * 2j)
-    assert_eq(d.dtype, complex)
-
-
-def test_dense_broadcasting():
-    shapeins = ((2,), (3, 2), (3, 1, 2))
-    shapeouts = ((3,), (2, 3), (2, 1, 3))
-    extradatas = ((), (4,), (2, 4))
-    extrainputs = ((), (5,), (3, 4))
-
-    def func(shapein, shapeout, extradata, extrainput, roll_input):
-        datashape = extradata + shapeout + shapein
-        inputshape = extrainput + shapein
-        d = np.arange(product(datashape)).reshape(datashape)
-        b = DenseOperator(d, naxesin=len(shapein), naxesout=len(shapeout),
-                          roll_input=roll_input, shapein=inputshape)
-        if roll_input:
-            assert b.shapeout == extrainput + extradata + shapeout
-        else:
-            assert b.shapeout == extradata + extrainput + shapeout
-        bdense = b.todense()
-        n = product(shapein)
-        if roll_input:
-            expected = BlockDiagonalOperator(
-                product(extrainput) *
-                [DenseOperator(d.reshape((-1, n)), shapein=n)],
-                axisin=0).todense()
-        else:
-            expected = BlockColumnOperator(
-                [BlockDiagonalOperator(product(extrainput) *
-                [DenseOperator(_.reshape((-1, n)), shapein=n)], axisin=0)
-                for _ in d.reshape((product(extradata), -1, n))],
-                axisout=0).todense()
-
-        assert_same(bdense, expected)
-        bTdense = b.T.todense()
-        assert_same(bTdense, bdense.T)
-
-        b2 = DenseOperator(d, naxesin=len(shapein), naxesout=len(shapeout),
-                           roll_input=roll_input)
-        assert_same(b2.todense(shapein=inputshape), bdense)
-        assert_same(b2.T.todense(shapein=b.T.shapein), bTdense)
-
-    for shapein in shapeins:
-        for shapeout in shapeouts:
-            for extradata in extradatas:
-                for extrainput in extrainputs:
-                    for roll_input in (False,): #(False, True):
-                        yield (func, shapein, shapeout, extradata, extrainput,
-                               roll_input)
-
-
-def test_dense_error():
-    shapes = ((2,), (3, 2), (4, 3, 2))
-    data = (np.arange(product(s)).reshape(s) for s in shapes)
-
-    def func(d):
-        b = DenseOperator(d)
-        assert_raises(ValueError, b, np.ones(3))
-    for d in data:
-        yield func, d
-
-
 #========================
 # Test ReductionOperator
 #========================
@@ -2493,24 +2468,39 @@ def test_asoperator_scalar():
 
 
 def test_asoperator_ndarray():
-    values = ([[1]], [[1, 2], [2, 3]], np.matrix([[3, 2.], [0, 1]]))
+    values = ([1], [2], [1, 2], [[1]], [[1, 2]], [[1, 2], [2, 3]],
+              [[[1, 2], [2, 3]]],
+              [[[1, 2], [2, 3]], [[4, 5], [6, 7]], [[8, 9], [10, 11]]])
+    cls = (IdentityOperator, HomothetyOperator, DiagonalOperator,
+           DenseOperator, DenseOperator)
 
-    def func1(v):
+    def totuple(seq):
+        if isinstance(seq, list):
+            return tuple(totuple(_) for _ in seq)
+        return seq
+
+    def func1(v, c, s):
         o = asoperator(v)
-        assert_is_instance(o, DenseOperator)
-        assert_eq(np.array(v).shape, o.shape)
+        assert_is_instance(o, c)
+        if len(s) > 1:
+            s = s[:-2] + (s[-1],)
+        assert_equal(o.shapein, s)
 
-    def func2(v):
+    def func2(v, s):
         o = asoperator(v, constant=True)
         if isinstance(v, np.matrix):
             assert_is_instance(o, DenseOperator)
-            assert_eq(np.array(v).shape, o.shape)
+            assert_equal(np.array(v).shape, o.shape)
         else:
             assert_is_instance(o, ConstantOperator)
-            assert_eq(np.array(v).shape, o.shapeout)
-    for v in values:
-        yield func1, v
-        yield func2, v
+            assert_equal(s, o.shapeout)
+    for v, c in zip(values, cls):
+        vt = totuple(v)
+        va = np.array(v)
+        s = va.shape
+        for data in v, vt, va:
+            yield func1, data, c, s
+            yield func2, data, s
 
 
 def test_asoperator_func():
@@ -2523,9 +2513,3 @@ def test_asoperator_func():
         assert_eq(o(v), f(np.array(v)))
     for v in (2, [2], [2, 3]):
         yield func, v
-
-
-def test_asoperator_ufunc():
-    assert_raises(TypeError, asoperator, np.maximum)
-    o = asoperator(np.cos)
-    assert_flags(o, 'real,inplace,square,separable')
